remove(list = ls(all=T))

setwd("C:/Users/goura/Desktop/DS Hub/Hackathon")

library(caret)

train_data=read.csv("train_LZdllcl.csv", header=TRUE, sep=",", na.strings ="")
test_data=read.csv("test_2umaH9m.csv", header=TRUE, sep=",", na.strings ="")

#OBJECTIVE: 
#Type of classification problem - binary classification
#Response variable is "is_promoted" 

################### 2. Understanding the data ########################################

dim(train_data)
dim(test_data)
setdiff(names(train_data),names(test_data))

summary(train_data)
str(train_data)
head(train_data)

summary(test_data)
str(test_data)
head(test_data)

# Understanding: - 
################### 3. Removing unnecessary columns ##################################
# Variables tobe discarded - 
train_data=subset(train_data, select=-c(employee_id))
test_id = test_data$employee_id
test_data=subset(test_data, select=-c(employee_id))

################### 5. Do necessary type conversions #################################
train_data$KPIs_met..80.=as.factor(train_data$KPIs_met..80.)
train_data$awards_won.=as.factor(train_data$awards_won.)
train_data$is_promoted=as.factor(train_data$is_promoted)

test_data$KPIs_met..80.=as.factor(test_data$KPIs_met..80.)
test_data$awards_won.=as.factor(test_data$awards_won.)

################### 6. Splitting the train data #######################################
library(caret)
set.seed(1234)
rows=createDataPartition(train_data$is_promoted,p = 0.7,list = FALSE)
train=train_data[rows,]
val=train_data[-rows,]

################### 7. Check NA values ###############################################
sum(is.na(train))
sum(is.na(val))
sum(is.na(test_data))

colSums(is.na(train))
colSums(is.na(val))
colSums(is.na(test_data))
str(train)

library(DMwR)
train = centralImputation(train)
val = centralImputation(val)
test_data = centralImputation(test_data)

################### 8. PreProcess the data to standardize the numeric attributes #####
test_data$is_promoted=""
preProc<-preProcess(train[,setdiff(names(train),"is_promoted")],method = c("center", "scale"))
train_std<-predict(preProc,train)
val_std<-predict(preProc,val)
test_std<-predict(preProc,test_data)

################### 9. Create dummies for factor varibales ###########################
dummies <- dummyVars(is_promoted~., data = train_std)

x.train=predict(dummies, newdata = train_std)
y.train=train_std$is_promoted

x.val = predict(dummies, newdata = val_std)
y.val = val_std$is_promoted

x.test = predict(dummies, newdata = test_std)

train_data1 = as.data.frame(cbind(x.train,y.train))
test_data1 = as.data.frame(x.test)
val_data1 = as.data.frame(cbind(x.val,y.val))

train_data1$y.train = ifelse(train_data1$y.train == 2, 1, 0)
val_data1$y.val = ifelse(val_data1$y.val == 2, 1, 0)

train_data1$y.train
################### 10. Check for clss imbalance  ###################################
table(train_data1$y.train)
prop.table(table(train_data1$y.train))
train_data1$y.train=as.factor(train_data1$y.train)

set.seed(2322)
train_smote<-SMOTE(y.train~. ,data=train_data1,perc.over = 700,perc.under = 100,k=3, learner=NULL)
table(train_smote$y.train)

prop.table(table(train_smote$y.train))
train_smote$y.train
#########################END OF DATA PRE-PROCESSING#################################

################### MODEL BUILDING #################################################
################### 1. Decision Tree(C5.0) #########################################
library(C50)

#Build model on smoteddata train_smote
DT_C50 <- C5.0(y.train~.,data=train_smote,rules=TRUE)
#DT_C50 <- C5.0(Revenue~.,data=trainsmote,rules=TRUE,trials=30)
summary(DT_C50)

##predict on train and validation
pred_train = predict(DT_C50,newdata=train_smote, type="class")
pred_val = predict(DT_C50, newdata=val_data1, type="class")
pred_test = predict(DT_C50, newdata=test_data1, type="class")

#Error Metrics on train and test
confusionMatrix(train_smote$y.train,pred_train)
confusionMatrix(as.factor(val_data1$y.val),pred_val)

test_c50 = cbind.data.frame('employee_id' = test_id, 'is_promoted' = pred_test)
write.csv(test_c50, file = 'Sub02_C50.csv', row.names = F)

################### 8. xgboost #####################################################
#Standardize all the real valued variables in the dataset using only the train data
#Convert data into an object of the class "xgb.Dmatrix", which works well with the xgboost model

library(xgboost)

train_matrix <- xgb.DMatrix(data = as.matrix(train_smote[, !(names(train_smote) %in% c("y.train"))]), 
                            label = as.matrix(train_smote[, names(train_smote) %in% "y.train"]))
val_matrix <- xgb.DMatrix(data = as.matrix(val_data1[, !(names(val_data1) %in% c("y.val"))]), 
                          label = as.matrix(val_data1[, names(val_data1) %in% "y.val"]))
test_matrix <- xgb.DMatrix(as.matrix(test_data1))

# Understanding XGBoost Parameters
modelLookup("xgbTree")

# Basic XGBoost Model

xgb_model_basic <- xgboost(data = train_matrix, max.depth = 2, 
                           eta = 1, nthread = 2, nround = 500, 
                           objective = "binary:logistic", verbose = 1, 
                           early_stopping_rounds = 10)
val_preds <- predict(xgb_model_basic, newdata=val_matrix)
val_preds_labels <- ifelse(val_preds < 0.5, 0, 1)
confusionMatrix(as.factor(val_preds_labels), as.factor(val_data1$y.val))

test_preds <- predict(xgb_model_basic, newdata=test_matrix)
test_preds_labels <- ifelse(test_preds < 0.5, 0, 1)

test_xgboost = cbind.data.frame('employee_id' = test_id, 'is_promoted' = test_preds_labels)
write.csv(test_xgboost, file = 'Sub02_xgboost.csv', row.names = F)

# Build an XGBoost Model with parameters

params_list <- list("objective" = "binary:logitraw",
                    "eta" = 0.1,
                    "early_stopping_rounds" = 10,
                    "max_depth" = 6,
                    "gamma" = 0.5,
                    "colsample_bytree" = 0.6,
                    "subsample" = 0.65,
                    "eval_metric" = "auc",
                    "silent" = 1)

xgb_model_with_params <- xgboost(data = train_matrix, params = params_list, nrounds = 500, early_stopping_rounds = 20)

val_params_preds <- predict(xgb_model_with_params, val_matrix)
val_params_preds_labels <- ifelse(val_params_preds < 0.5, 0, 1)
confusionMatrix(as.factor(val_params_preds_labels), as.factor(val_data1$y.val))

test_preds <- predict(xgb_model_with_params, newdata=test_matrix)
test_preds_labels <- ifelse(test_preds < 0.5, 0, 1)

test_xgboost_params = cbind.data.frame('employee_id' = test_id, 'is_promoted' = test_preds_labels)
write.csv(test_xgboost_params, file = 'Sub03_xgboost.csv', row.names = F)

## Plotting Variable Importance

variable_importance_matrix <- xgb.importance(feature_names = colnames(train_matrix), model = xgb_model_with_params)

xgb.plot.importance(variable_importance_matrix)

################## Tuning an XGBoost Model with the caret package ############################

modelLookup("xgbTree")

sampling_strategy <- trainControl(method = "cv", number = 5, verboseIter = T, allowParallel = T)

param_grid <- expand.grid(.nrounds = 50, .max_depth = c(2, 4, 6), .eta = c(0.05,0.1),
                          .gamma = c(0.6, 0.3), .colsample_bytree = c(0.6, 0.8),
                          .min_child_weight=1,.subsample = c(0.8, 0.9))

xgb_tuned_model <- train(x = train_data1[ , !(names(train_smote) %in% c("y.train"))], 
                         y = train_data1[ , names(train_smote) %in% c("y.train")], 
                         method = "xgbTree",
                         trControl = sampling_strategy,
                         metrics="Accuracy",
                         objective="binary:logistic",
                         tuneGrid = param_grid)
train_data1$y.train

xgb_tuned_model$bestTune

View(xgb_tuned_model$results)

tuned_params_train_preds <- predict(xgb_tuned_model, train_data1[ , !(names(train_data1) %in% c("y.train"))])

tuned_params_val_preds <- predict(xgb_tuned_model, val_data1[ , !(names(val_data1) %in% c("y.val"))])

confusionMatrix(as.factor(tuned_params_train_preds), as.factor(train_data1$y.train))

confusionMatrix(as.factor(tuned_params_val_preds), as.factor(val_data1$y.val))

test_preds <- predict(xgb_tuned_model, newdata=test_data1)
test_preds_labels <- ifelse(test_preds < 0.5, 0, 1)

test_xgboost__tuned_params = cbind.data.frame('employee_id' = test_id, 'is_promoted' = test_preds)
write.csv(test_xgboost__tuned_params, file = 'Sub04_xgboost.csv', row.names = F)

################### 9. Stacking #############################################################
